=== modified file 'src/BamfLauncherIcon.cpp'
--- src/BamfLauncherIcon.cpp	2011-01-31 13:54:38 +0000
+++ src/BamfLauncherIcon.cpp	2011-02-01 10:26:05 +0000
@@ -51,6 +51,13 @@
   indicator_desktop_shortcuts_nick_exec (data->shortcuts, data->nick);
 }
 
+void
+BamfLauncherIcon::Activate ()
+{
+  std::cout << "BamfLauncherIcon::Activate()" << std::endl;
+  OpenInstance ();
+}
+
 BamfLauncherIcon::BamfLauncherIcon (Launcher* IconManager, BamfApplication *app, CompScreen *screen)
 :   SimpleLauncherIcon(IconManager)
 {

=== modified file 'src/BamfLauncherIcon.h'
--- src/BamfLauncherIcon.h	2011-01-14 18:17:34 +0000
+++ src/BamfLauncherIcon.h	2011-02-01 10:26:05 +0000
@@ -41,6 +41,7 @@
 
     const char* DesktopFile ();
     bool IsSticky ();
+    void Activate ();
 
 protected:
     void OnMouseClick (int button);

=== modified file 'src/Launcher.cpp'
--- src/Launcher.cpp	2011-01-31 14:36:13 +0000
+++ src/Launcher.cpp	2011-02-01 10:26:05 +0000
@@ -187,6 +187,8 @@
     OnMouseLeave.connect (sigc::mem_fun (this, &Launcher::RecvMouseLeave));
     OnMouseMove.connect  (sigc::mem_fun (this, &Launcher::RecvMouseMove));
     OnMouseWheel.connect (sigc::mem_fun (this, &Launcher::RecvMouseWheel));
+    OnKeyPressed.connect (sigc::mem_fun (this, &Launcher::RecvKeyPressed));
+    OnKeyReleased.connect (sigc::mem_fun (this, &Launcher::RecvKeyReleased));
 
     QuicklistManager::Default ()->quicklist_opened.connect (sigc::mem_fun(this, &Launcher::RecvQuicklistOpened));
     QuicklistManager::Default ()->quicklist_closed.connect (sigc::mem_fun(this, &Launcher::RecvQuicklistClosed));
@@ -205,6 +207,11 @@
     m_ActiveMenuIcon = NULL;
     m_LastSpreadIcon = NULL;
 
+    _current_icon       = NULL;
+    _last_selected_icon = NULL;
+    _current_icon_index = -1;
+    _last_icon_index    = -1;
+
     SetCompositionLayout(m_Layout);
 
     if(nux::GetGraphicsEngine ().UsingGLSLCodePath ())
@@ -2056,6 +2063,97 @@
   EnsureAnimation ();
 }
 
+void Launcher::RecvKeyPressed (unsigned int  key_sym, unsigned long key_code,
+                               unsigned long key_state)
+{
+  /*fprintf (stdout,
+           "Launcher::RecvKeyPressed (%d, %ld, %ld)\n",
+           key_sym,
+           key_code,
+           key_state);*/
+}
+
+void Launcher::UpdateKeyNavIndicator ()
+{
+  std::cout << "UpdateKeyNavIndicator() called" << std::endl;
+}
+
+void Launcher::RecvKeyReleased (unsigned int  key_sym, unsigned long key_code,
+                                unsigned long key_state)
+{
+  std::cout << "Launcher::RecvKeyReleased() called" << std::endl;
+
+  switch (key_sym)
+  {
+    // up (move selection up or go to global-menu if at top-most icon)
+    case XK_Up:
+      std::cout << "Up" << std::endl;
+      if (_current_icon_index > 0)
+        _current_icon_index--;
+      else
+      {
+        _current_icon_index = -1;
+        // switch to global-menu
+      }
+      UpdateKeyNavIndicator ();
+    break;
+
+    // down (move selection down and unfold launcher if needed)      
+    case XK_Down:
+      std::cout << "Down" << std::endl;
+      if (_current_icon_index < _model->Size ())
+      {
+        _current_icon_index++;
+        UpdateKeyNavIndicator ();
+      }
+    break;
+
+    // esc/left (close quicklist or exit laucher key-focus)
+    case XK_Left:
+    case XK_Escape:
+      std::cout << "ESC/Left" << std::endl;
+      // hide again
+      _last_icon_index = _current_icon_index;
+    break;
+
+    // right/shift-f10 (open quicklist of currently selected icon)      
+    case XK_Right:
+    case XK_F10:
+      std::cout << "Shift-F10/Right" << std::endl;
+      // un-hide
+      if (_last_icon_index >= 0)
+        _current_icon_index = _last_icon_index;
+      else
+        _current_icon_index = 0;
+      UpdateKeyNavIndicator ();
+    break;
+
+    // <RETURN>/<SPACE> (start/activate currently selected icon)      
+    case XK_space:
+    case XK_Return:
+      std::cout << "RETURN/SPACE" << std::endl;
+      // start current selected icon
+      if (_current_icon_index < _model->Size ())
+      {
+        std::cout << "icon-index:" << _current_icon_index << std::endl;
+        LauncherModel::iterator it;
+        int i;
+        for (it = _model->begin (), i = 0; it != _model->end (), i <= _current_icon_index; it++, i++);
+        std::cout << "name: " << (*it)->GetName () << std::endl;
+        (*it)->Activate ();
+      }
+      else
+      {
+        _current_icon_index = _model->Size () - 1;
+        UpdateKeyNavIndicator ();
+      }
+    break;
+
+    default:
+    break;
+  }
+}
+
 void Launcher::RecvQuicklistOpened (QuicklistView *quicklist)
 {
   EventLogic ();

=== modified file 'src/Launcher.h'
--- src/Launcher.h	2011-01-31 13:55:03 +0000
+++ src/Launcher.h	2011-02-01 10:26:05 +0000
@@ -109,6 +109,9 @@
   virtual void RecvMouseMove(int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags);
   virtual void RecvMouseWheel(int x, int y, int wheel_delta, unsigned long button_flags, unsigned long key_flags);
 
+  virtual void RecvKeyPressed (unsigned int key_sym, unsigned long key_code, unsigned long key_state);
+  virtual void RecvKeyReleased (unsigned int key_sym, unsigned long key_code, unsigned long key_state);
+
   virtual void RecvQuicklistOpened (QuicklistView *quicklist);
   virtual void RecvQuicklistClosed (QuicklistView *quicklist);
 
@@ -270,6 +273,8 @@
   void SetOffscreenRenderTarget (nux::IntrusiveSP<nux::IOpenGLBaseTexture> texture);
   void RestoreSystemRenderTarget ();
 
+  void UpdateKeyNavIndicator ();
+
   nux::HLayout* m_Layout;
   int m_ContentOffsetY;
 
@@ -277,6 +282,12 @@
   LauncherIcon* m_ActiveMenuIcon;
   LauncherIcon* m_LastSpreadIcon;
 
+  // used by keyboard/a11y-navigation
+  LauncherIcon* _current_icon;
+  LauncherIcon* _last_selected_icon;
+  int _current_icon_index;
+  int _last_icon_index;
+
   QuicklistView* _active_quicklist;
 
   bool  _hovered;

=== modified file 'src/LauncherIcon.cpp'
--- src/LauncherIcon.cpp	2011-01-28 15:20:23 +0000
+++ src/LauncherIcon.cpp	2011-02-01 10:26:05 +0000
@@ -131,6 +131,12 @@
   g_variant_builder_add (builder, "{sv}", "quirk-presented", g_variant_new_boolean (GetQuirk (QUIRK_PRESENTED)));
 }
 
+void
+LauncherIcon::Activate ()
+{
+  std::cout << "LauncherIcon::Activate() called" << std::endl;
+}
+
 nux::Color LauncherIcon::BackgroundColor ()
 {
   return _background_color;

=== modified file 'src/LauncherIcon.h'
--- src/LauncherIcon.h	2011-01-28 15:20:23 +0000
+++ src/LauncherIcon.h	2011-02-01 10:26:05 +0000
@@ -91,7 +91,9 @@
     
     void        SetCenter (nux::Point3 center);
     nux::Point3 GetCenter ();
-    
+
+    virtual void Activate ();
+
     void SaveCenter ();
     
     int SortPriority ();

=== modified file 'src/SimpleLauncherIcon.cpp'
--- src/SimpleLauncherIcon.cpp	2011-01-13 15:54:59 +0000
+++ src/SimpleLauncherIcon.cpp	2011-02-01 10:26:05 +0000
@@ -44,6 +44,12 @@
 }
 
 void
+SimpleLauncherIcon::Activate ()
+{
+  std::cout << "SimpleLauncherIcon::Activate() called" << std::endl;
+}
+
+void
 SimpleLauncherIcon::OnMouseDown (int button)
 {
 }

=== modified file 'src/SimpleLauncherIcon.h'
--- src/SimpleLauncherIcon.h	2011-01-13 15:54:59 +0000
+++ src/SimpleLauncherIcon.h	2011-02-01 10:26:05 +0000
@@ -34,6 +34,7 @@
     nux::BaseTexture * GetTextureForSize (int size);
     
     void SetIconName (const char *name);
+    virtual void Activate ();
 
 protected:
     virtual void OnMouseDown (int button);

=== modified file 'src/unityshell.cpp'
--- src/unityshell.cpp	2011-01-31 13:55:03 +0000
+++ src/unityshell.cpp	2011-02-01 10:26:05 +0000
@@ -212,6 +212,31 @@
   return false;
 }
 
+bool
+UnityScreen::setKeyboardFocusKeyInitiate (CompAction         *action,
+                                          CompAction::State  state,
+                                          CompOption::Vector &options)
+{
+  CompWindow* w = screen->findWindow (launcherWindow->GetInputWindowId ());
+    
+  // to receive the Terminate event
+  /*if (state & CompAction::StateInitKey)
+    action->setState (action->state () | CompAction::StateTermKey);*/
+
+  if (w != NULL)
+    w->moveInputFocusTo ();
+
+  return false;
+}
+
+/*bool
+UnityScreen::setKeyboardFocusKeyTerminate (CompAction         *action,
+                                           CompAction::State   state,
+                                           CompOption::Vector &options)
+{
+  return false;
+}*/
+
 gboolean
 UnityScreen::initPluginActions (gpointer data)
 {
@@ -497,7 +522,9 @@
   optionSetUrgentAnimationNotify   (boost::bind (&UnityScreen::optionChanged, this, _1, _2));
   optionSetShowLauncherInitiate (boost::bind (&UnityScreen::showLauncherKeyInitiate, this, _1, _2, _3));
   optionSetShowLauncherTerminate (boost::bind (&UnityScreen::showLauncherKeyTerminate, this, _1, _2, _3));
-
+  optionSetKeyboardFocusInitiate (boost::bind (&UnityScreen::setKeyboardFocusKeyInitiate, this, _1, _2, _3));
+  //optionSetKeyboardFocusTerminate (boost::bind (&UnityScreen::setKeyboardFocusKeyTerminate, this, _1, _2, _3));
+    
   g_timeout_add (0, &UnityScreen::initPluginActions, this);
   g_timeout_add (5000, (GSourceFunc) write_logger_data_to_disk, NULL);
   END_FUNCTION ();

=== modified file 'src/unityshell.h'
--- src/unityshell.h	2011-01-20 04:06:54 +0000
+++ src/unityshell.h	2011-02-01 10:26:05 +0000
@@ -100,6 +100,16 @@
 	showLauncherKeyTerminate (CompAction *action, CompAction::State state,
                               CompOption::Vector &options);
 
+  bool
+  setKeyboardFocusKeyInitiate (CompAction*         action,
+                               CompAction::State   state,
+                               CompOption::Vector& options);
+
+  /*bool
+  setKeyboardFocusKeyTerminate (CompAction*        action,
+                                CompAction::State  state,
+                                CompOption::Vector& options);*/
+
 	/* handle option changes and change settings inside of the
 	 * panel and dock views */
 	void optionChanged (CompOption *, Options num);

=== modified file 'unityshell.xml.in'
--- unityshell.xml.in	2011-01-04 16:16:10 +0000
+++ unityshell.xml.in	2011-02-01 10:26:05 +0000
@@ -49,6 +49,11 @@
                     <_long>Make the launcher appear with that key</_long>
                     <default>&lt;Super&gt;</default>
                   </option>
+                  <option name="keyboard_focus" type="key">
+		    <_short>Key to put keyboard-focus on launcher</_short>
+		    <_long>Set the keyboard-focus on the launcher so it can be navigated with the cursor-keys</_long>
+		    <default>&lt;Alt&gt;F1</default>
+                  </option>
 	          </group>
 	          <group>
                 <_short>Experimental</_short>

